/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <drivers/gpio.h>
#include <zephyr/sys/heap_listener.h>

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   5000

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)


void main_heap_listener_alloc_cbh(uintptr_t heap_id, void *mem, size_t bytes);
void main_heap_listener_free_cbh(uintptr_t heap_id, void *mem, size_t bytes);

//**SYSHP static struct sys_heap listener_heap;
//**SYSHP static void *heapmem[1024];

/* using defined system heap */
//**SYSHP HEAP_LISTENER_ALLOC_DEFINE(AllocListener, HEAP_ID_FROM_POINTER(&listener_heap), main_heap_listener_alloc_cbh);
//**SYSHP HEAP_LISTENER_FREE_DEFINE(FreeListener, HEAP_ID_FROM_POINTER(&listener_heap), main_heap_listener_free_cbh);

/* using default system heap */
HEAP_LISTENER_ALLOC_DEFINE(AllocListener, HEAP_ID_LIBC, main_heap_listener_alloc_cbh);
HEAP_LISTENER_FREE_DEFINE(FreeListener, HEAP_ID_LIBC, main_heap_listener_free_cbh);

void main_heap_listener_alloc_cbh(uintptr_t heap_id, void *mem, size_t bytes)
{
    printk("=> Allocate %i <=\n", bytes);
} // main_heap_listener_alloc_cbh()

void main_heap_listener_free_cbh(uintptr_t heap_id, void *mem, size_t bytes)
{
    printk("=> Free %i <=\n", bytes);
} // main_heap_listener_free_cbh()

/*
 * A build error on this line means your board is unsupported.
 * See the sample documentation for information on how to fix this.
 */
static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

void main(void)
{
	int ret;

    /* init system heap */
//**SYSHP     sys_heap_init(&listener_heap, heapmem, 1024);
	
    /* Register heap listeners */
	heap_listener_register(&AllocListener);
	heap_listener_register(&FreeListener);    

    printk("Registered heap listeners..\n");

	if (!device_is_ready(led.port)) {
		return;
	}

	ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		return;
	}

//**SYSHP     void *mem = sys_heap_alloc(&listener_heap, 32U);
    void *mem = k_malloc(32);  /* default heap */

    char *str = (char*)mem;
    str[0] = 'h';

//**SYSHP     sys_heap_free(&listener_heap, mem);
    k_free(mem);        /* default heap */

	while (1) {
		ret = gpio_pin_toggle_dt(&led);
		if (ret < 0) {
			return;
		}
		k_msleep(SLEEP_TIME_MS);
	}
}
